﻿using ProjectShoeShop.Reports;
using ProjectShoeShop.Repositories;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Unity;

namespace ProjectShoeShop.Forms
{
    public partial class FormShoesReport : Form
    {
        private readonly IUnityContainer _container;

        public FormShoesReport(IUnityContainer container, IShoesRepository shoesRepository)
        {
            InitializeComponent();

            _container = container ?? throw new ArgumentNullException(nameof(container));

            comboBoxShoes.DataSource = shoesRepository.ReadShoes();
            comboBoxShoes.DisplayMember = "Name";
            comboBoxShoes.ValueMember = "Id";
        }

        private void buttonFile_Click(object sender, EventArgs e)
        {
            var sfd = new SaveFileDialog()
            {
                Filter = "Excel Files | *.xlsx"
            };
            if (sfd.ShowDialog() != DialogResult.OK)
            {
                return;
            }
            textBoxFilePath.Text = sfd.FileName;
        }

        private void buttonBuild_Click(object sender, EventArgs e)
        {
            try
            {
                if (string.IsNullOrWhiteSpace(textBoxFilePath.Text))
                {
                    throw new Exception("Отсутствует имя файла для отчета");
                }
                if (comboBoxShoes.SelectedIndex < 0)
                {
                    throw new Exception("Не выбрана обувь");
                }
                if (dateTimePickerEnd.Value <= dateTimePickerStart.Value)
                {
                    throw new Exception("Дата начала должна быть раньше даты окончания");
                }
                if (_container.Resolve<TableReport>().CreateTable(textBoxFilePath.Text, (int)comboBoxShoes.SelectedValue!, dateTimePickerStart.Value, dateTimePickerEnd.Value))
                {
                    MessageBox.Show("Документ сформирован",
                    "Формирование документа",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Information);
                }
                else
                {
                    MessageBox.Show("Возникли ошибки при формировании документа.Подробности в логах",
                    "Формирование документа",
                    MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Ошибка при создании очета",
                MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
    }
}
