using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.Logging;
using ProjectShoeShop.Repositories;
using ProjectShoeShop.Repositories.Implementations;
using Serilog;
using Unity;
using Unity.Microsoft.Logging;

namespace ProjectShoeShop
{
    internal static class Program
    {
        /// <summary>
        ///  The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main()
        {
            // To customize application configuration such as set high DPI settings or default font,
            // see https://aka.ms/applicationconfiguration.
            ApplicationConfiguration.Initialize();
            Application.Run(CreateContainer().Resolve<FormShoesShop>());
        }

        private static IUnityContainer CreateContainer()
        {
            var container = new UnityContainer();
            container.AddExtension(new LoggingExtension(CreateLoggerFactory()));

            container.RegisterType<IClientRepository, ClientRepository>();
            container.RegisterType<IOrderRepository, OrderRepository>();
            container.RegisterType<IShoesRepository, ShoesRepository>();
            container.RegisterType<IConnectionString, ConnectionString>();
            container.RegisterType<ISupplyRepository, SupplyRepository>();
        
            return container;
        }
        private static LoggerFactory CreateLoggerFactory()
        {
            var loggerFactory = new LoggerFactory();
            loggerFactory.AddSerilog(new LoggerConfiguration()
                .ReadFrom.Configuration(new ConfigurationBuilder()
                .SetBasePath(Directory.GetCurrentDirectory())
                .AddJsonFile("appsettings.json")
                .Build())
                .CreateLogger());
            return loggerFactory;
        }
    }
}