﻿using Dapper;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using Npgsql;
using ProjectShoeShop.Entities;

namespace ProjectShoeShop.Repositories.Implementations;

public class ClientRepository : IClientRepository
{
    private readonly IConnectionString _connectionString;
    private readonly ILogger<ClientRepository> _logger;

    public ClientRepository(IConnectionString connectionString, ILogger<ClientRepository> logger)
    {
        _connectionString = connectionString;
        _logger = logger;
    }

    public void CreateClient(Client client)
    {
        _logger.LogInformation("Добавление объекта");
        _logger.LogDebug("Объект: {json}", JsonConvert.SerializeObject(client));

        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var queryInsert = @"
            INSERT INTO Client (Name)
            VALUES (@Name)";
            connection.Execute(queryInsert, client);
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при добавлении объекта");
            throw;
        }
    }
 
    public void DeleteClient(int id)
    {
        _logger.LogInformation("Удаление объекта");
        _logger.LogDebug("Объект: {id}", id);
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var queryDelete = @"
                DELETE FROM Client
                WHERE Id=@id";
            connection.Execute(queryDelete, new { id });
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при удалении объекта");
            throw;
        }
    }

    public IEnumerable<Client> ReadClient()
    {
        _logger.LogInformation("Получение всех объектов");
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var querySelect = "SELECT * FROM Client";
            var clients = connection.Query<Client>(querySelect);
            _logger.LogDebug("Полученные объекты: {json}",
            JsonConvert.SerializeObject(clients));
            return clients;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при чтении объектов");
            throw;
        }
    }

    public Client ReadClientById(int id)
    {
        _logger.LogInformation("Получение объекта по идентификатору");
        _logger.LogDebug("Объект: {id}", id);
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var querySelect = @"
                SELECT * FROM Client
                WHERE Id=@id";
            var client = connection.QueryFirst<Client>(querySelect, new { id });
            _logger.LogDebug("Найденный объект: {json}",
                JsonConvert.SerializeObject(client));
            return client;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при поиске объекта");
            throw;
        }
    }

    public void UpdateClient(Client client)
    {
        _logger.LogInformation("Редактирование объекта");
        _logger.LogDebug("Объект: {json}",
            JsonConvert.SerializeObject(client));
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var queryUpdate = @"
                UPDATE Client
                SET
                Name=@Name
                WHERE Id=@Id";
            connection.Execute(queryUpdate, client);
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при редактировании объекта");
            throw;
        }
    }
}
